@@ -39,11 +39,13 @@ class FoldersPanel : public Panel
     Q_OBJECT
 
 public:
-    FoldersPanel(QWidget* parent = 0);
+    explicit FoldersPanel(QWidget* parent = nullptr);
     virtual ~FoldersPanel();
 
     void setShowHiddenFiles(bool show);
+    void setLimitFoldersPanelToHome(bool enable);
     bool showHiddenFiles() const;
+    bool limitFoldersPanelToHome() const;
 
     void setAutoScrolling(bool enable);
     bool autoScrolling() const;
@@ -81,6 +83,7 @@ private slots:
      */
     void startFadeInAnimation();
 
+
 private:
     /**
      * Initializes the base URL of the tree and expands all
@@ -89,6 +92,8 @@ private:
      */
     void loadTree(const QUrl& url);
 
+    void reloadTree();
+
     /**
      * Sets the item with the index \a index as current item, selects
      * the item and assures that the item will be visible.
